package epsilonStandalone

import java.io.BufferedReader
import java.io.File
import java.io.FileWriter
import java.io.InputStreamReader
import java.net.URI
import java.net.URISyntaxException
import java.text.DecimalFormat
import java.text.SimpleDateFormat
import java.util.Date
import java.util.HashMap
import java.util.LinkedHashMap
import java.util.List
import java.util.Map
import org.eclipse.epsilon.common.parse.problem.ParseProblem
import org.eclipse.epsilon.common.util.StringProperties
import org.eclipse.epsilon.egl.EglFileGeneratingTemplateFactory
import org.eclipse.epsilon.egl.EglTemplateFactoryModuleAdapter
import org.eclipse.epsilon.emc.emf.EmfModel
import org.eclipse.epsilon.eol.EolModule
import org.eclipse.epsilon.eol.IEolModule
import org.eclipse.epsilon.eol.exceptions.EolRuntimeException
import org.eclipse.epsilon.eol.exceptions.models.EolModelLoadingException
import org.eclipse.epsilon.eol.execute.context.Variable
import org.eclipse.epsilon.eol.models.IModel
import org.eclipse.epsilon.eol.models.IRelativePathResolver
import org.eclipse.epsilon.eol.types.EolPrimitiveType
import org.eclipse.epsilon.etl.EtlModule

@Data class ExperimentModel {
	String experimentName
	String modelURI
}
	
	
public class DNIRunner{
	
	
	//open this script and set the paths!!
	var protected String simqpn = "bash /home/mk/workspaces/workspace_DNI_neo/DNI3/DNIinstances/simutools2016/results/runSimQPN.sh -r simutools2016 "
	var protected simulationSilent = true
	var protected kiloMultipler = 1000
	
	var protected pwdUnix = System.getProperty("user.dir")+"/";//"/Users/Piotr/WuePhD/Research/DNIcode/xtend-epsilon-standalone/"
	var protected pwdURI = "file:"+pwdUnix

	var protected dniModelsDirRelative = "src/epsilonStandalone/models/dni/"
	var protected tmpModelsDirRelative = "src/epsilonStandalone/models/tmp/"
	var protected qpmeModelsDirRelative = "src/epsilonStandalone/models/qpme/"
	var protected qpmeResultsDirRelative = "src/epsilonStandalone/models/qpmeResults/"
	
	var protected dni_base_model_file = "models/scenario0A.model"
	
	val protected dniModels = newLinkedList()
	val protected qpmeModels = newLinkedList()
	val protected qpmeFiles = newLinkedList()
	val protected qpmeSimulationResultFiles = newLinkedList()
	val protected qpmeSimulationProcessedResults = newLinkedList()
	
	var protected simQpnParser = new SimQpnResultsParser()
	
	
	var protected HashMap<String, List<Variable>> epsilonTransformationParameters = new HashMap <String, List<Variable>>()
	
	
	var protected currentTransformationFile = "etl/standalone-DNI-QPN-new.etl"
//	var currentTransformationFile = "etl/DNI-QPN.etl"
  	
	
	public def static void main(String[] args) {
	  	
	  	new DNIRunner().start_single()	
	}
	
	
	//DEMONSTRATION HOW TO USE THIS CLASS
	public def start_single(){
		simQpnParser.interestingPlaces = newLinkedList("SW35")//, "SW_SDN_Switching_QP_SwitchSDN(HP3500)", "HW_SDN_Switching_QP_SwitchSDN(HP3500)")
  		simQpnParser.interestingColors = newLinkedList("Traffic")
  		simQpnParser.interestingMetrics = newLinkedList("arrivThrPut")//, "deptThrPut")

		epsilonTransformationParameters.clear
		var expNamePrefix = "AA_Debug_run";
		var String experimentName = expNamePrefix;
		
		
		
		var ExperimentModel pairListExpNameDniModel;
		pairListExpNameDniModel = stateless_applyEolStriptToBaseModel(dni_base_model_file, "eol/simutools_check.eol", newLinkedList(), 0, experimentName);
		//pairListExpNameDniModel = String experimentName -> String DNI model URI
		var dniModelURI = pairListExpNameDniModel.modelURI
		
		var List<Variable> paramList = newLinkedList(new Variable("param_goOverSdnSW", true, EolPrimitiveType.Boolean))
		
		
													
		var outFileName = experimentName+".qpmemodel"
		var destinationModelURI = pwdURI+qpmeModelsDirRelative+outFileName
		println("Will store transformation result in: "+destinationModelURI)
		
		destinationModelURI = "file:models/qpme/demo.qpmemodel"
		var String outputQpnURI = stateless_applyEtlStript("DNI", dniModelURI, "metamodels/dnimm3.ecore", 
															"QPME", destinationModelURI, "metamodels/qpme.ecore",
															currentTransformationFile, paramList, experimentName)
															
		var qpeFileContents = stateless_applyEglScript("QPME", outputQpnURI, "metamodels/qpme.ecore", "egl/dni2qpme.egl", experimentName)
		
		var outputQpeFilePath = pwdUnix+qpmeModelsDirRelative+experimentName+".qpe"  //= AA_Debug_run.qpe
		saveOutput(qpeFileContents, outputQpeFilePath)	
		
		println("Your .qpe file is here: "+outputQpeFilePath)
		
		//TODO: stateless simulate
		//TODO: stateless analyze simulation results
		
		///cleanup - remove files *.result.xml, *FlatHQPN*.qpe and so on
		cleanup
		
	}
	

	
	
	
	private def boolean deleteFile(String path){
		var File f = null
		f = new File(path);
      	return f.delete();
        
	}
	
	public static def String getCurrentTimeStamp() {
    	var SimpleDateFormat sdfDate = new SimpleDateFormat("yyy.MM.dd.HH.mm.ss");
    	var Date now = new Date();
    	var String strDate = sdfDate.format(now);
    	return strDate;
	}
	
	private def List<String> getFileList(String directory, String endsWith, String contains){
		
		var List<String> files = newLinkedList()
		
		var File folder = new File(directory);
		//println("Reading dir " + directory);
		
		var File[] listOfFiles = folder.listFiles();
		for(f : listOfFiles){
			var File file = f as File
			
			if (file.isFile()) {
				var String fname = file.getName(); 
				//System.out.println("File " + fname);
				if(fname.endsWith(endsWith) && !endsWith.isEmpty && contains.isEmpty){
					var String path = directory+"/"+fname; 
					files.add(path);
				}
				else if(fname.contains(contains) && !contains.isEmpty && !contains.isEmpty){
					var String path = directory+"/"+fname; 
					files.add(path);
				}
				else{
					if(fname.contains(contains) && fname.endsWith(endsWith)){
						var String path = directory+"/"+fname; 
						files.add(path);	
					}
				}
			}
		}
		return files
	}
		
	
	
	
	protected def parseAllResults(String finalFileName){
		
		var String finalResult = ""
		for(resultFile : qpmeSimulationResultFiles){
						
			var String experimentName = (resultFile as ExperimentModel).experimentName
			var String qpeResultFile = (resultFile as ExperimentModel).modelURI
			
			var String result = parseResult(qpeResultFile, experimentName)
			var String line = experimentName+" # "+result
			finalResult += line
			println(line)
			qpmeSimulationProcessedResults.add(result) 
			
			
		}
		
		saveOutput(finalResult, pwdUnix+qpmeResultsDirRelative+finalFileName)		
	}
	
	public def String parseResult(String qpeResultFile, String experimentName){
		//ANALYZE RESULTS
		var String toReturn =""//here in experiment 3 we will return throughput of the receiver in Mbps
		
		if(new File(qpeResultFile).isFile() == false){
			return " File does not exist."
		}
	  	
	  	var HashMap<String, HashMap<String, Double>> summary = new HashMap<String, HashMap<String, Double>>();
	  	
	  	var HashMap<String, HashMap<String, LinkedHashMap<String, Double>>> map = simQpnParser.parse(qpeResultFile);
	  	
		var i1 = map.entrySet().iterator();
		
	    while(i1.hasNext()) {
	         var me = i1.next() as Map.Entry<String, HashMap<String, LinkedHashMap<String, Double>>>
	         var placeName = me.key as String
	         var colorMetricValueHM = me.value as HashMap<String, LinkedHashMap<String, Double>>
	         
	         var i2 = colorMetricValueHM.entrySet().iterator()
	         while(i2.hasNext()) {
	         	var me2 = i2.next() as Map.Entry<String, LinkedHashMap<String, Double>>
	         	var colorName = me2.key as String
	         	var metricValueHM = me2.value as LinkedHashMap<String, Double>
	         	
	         	var i3 = metricValueHM.entrySet().iterator()
	         	while(i3.hasNext()) {
	         		var me3 = i3.next() as Map.Entry<String, Double>;
	         		var metricName = me3.key as String
	         		var double value = me3.value as Double
	         		
	         		var double tokensPerSec = value
					var double bytes = simQpnParser.parseColorMessageSize(colorName, kiloMultipler);	
					var double bytesPerSec = tokensPerSec * bytes;
					var double bitsPerSec = bytesPerSec*8
					var double MbitsPerSec = bitsPerSec/1000/1000 
					
					var String MbitsPerSecTxt = new DecimalFormat("#.##").format(MbitsPerSec);
					/*
					toReturn += "PLACE "+simQpnParser.getFriendlyName(placeName)
					//FIXME: uncomment for general use!
					toReturn += " COLOR "+colorName
					toReturn += " METRIC "+metricName
					toReturn += " VALUE "+MbitsPerSecTxt+" Mbps"
					//FIXME: uncomment for general use!
					toReturn += " ("+tokensPerSec+" Tokensps)"
					toReturn += "\n"
					*/
					if (!summary.containsKey(placeName)) {
						summary.put(placeName, new HashMap<String, Double>());
					}
					summary.get(placeName).put(colorName, MbitsPerSec);					
					
	         	}
	         }
         }
         
         
         //toReturn += "\n======================================================================\nSUMMARY:\n";
         var keyList = summary.keySet().toList();
         java.util.Collections.sort(keyList);
         for (k : keyList) {
         	toReturn += simQpnParser.getFriendlyName(k) +" --- "
         	var innerList = summary.get(k).keySet().toList();
         	java.util.Collections.sort(innerList);
         	
         	var sum = 0.0
         	for (kk : innerList) {
         		sum += summary.get(k).get(kk);        		
         	} 
         	toReturn += sum + " Mbps\n";   	
         }
         
         toReturn
	}
	
	protected def simulateAll(){
		var int i =0
		for(qpmeFile : qpmeFiles){
			i +=1
			println("Simulating run #"+i+" of "+qpmeFiles.size)
			var experimentName = (qpmeFile as ExperimentModel).experimentName
			var qpeFilePath = (qpmeFile as ExperimentModel).modelURI
			var String resultsFilePath = simulateSimQPN(qpeFilePath, experimentName)
			println("Adding " + experimentName + " with path " + qpeFilePath + " to qpeSimulationResultFiles")
			qpmeSimulationResultFiles.add(new ExperimentModel(experimentName, resultsFilePath))
		}
		

//	bash 
//	/Users/Piotr/WuePhD/Research/DNIcode/DNI3/DNIinstances/simutools2016/results/runSimQPN.sh -r simutools2016 
//	/Users/Piotr/WuePhD/Research/DNIcode/xtend-epsilon-standalone/src/epsilonStandalone/models/qpme/Experiment3_mtu_1188_latency_0.02_pps_10000.qpe 
//	-o file:
//	/Users/Piotr/WuePhD/Research/DNIcode/xtend-epsilon-standalone/src/epsilonStandalone/models/qpmeResults/Experiment3_mtu_1188_latency_0.02_pps_10000.results.xml

	}
	
	private def String simulateSimQPN(String qpeFile, String experimentName){

	  	
	  	
	  	var simulation_output_file = "/home/mk/workspaces/workspace_DNI_neo/DNI3/DNIinstances/"+experimentName+".results.xml"
	  	simulation_output_file = simulation_output_file.replace("file:/","/")
	  	
	  	var output_simulation = " -v /dev/null -o "+simulation_output_file
	
	  	
	  	var String command = simqpn + qpeFile + output_simulation;
	  	
  		println("COMMAND: "+command)
	  	var Process p = Runtime.getRuntime().exec(command);
	  	
	  	
	  	
	  	//reading output of system command
		var BufferedReader stdInput = new BufferedReader(new InputStreamReader(p.getInputStream()));
		var BufferedReader stdError = new BufferedReader(new InputStreamReader(p.getErrorStream()));
		var s = stdInput.readLine();
		while (s != null) { if(!simulationSilent){println(s);} s = stdInput.readLine();}
	    var se = stdError.readLine();
	  	while (se != null) { if(!simulationSilent){println(se);} se = stdError.readLine();}
	  	
	  	return simulation_output_file

	}
		
	/*
	 * Takes qpme model and returns the contents of the qpe file
	 */
	private def String modelToTextQpn(String sourceURI, String experimentName){
		
		return stateless_applyEglScript("QPME", sourceURI, "models/qpme.ecore", "egl/dni2qpme.egl", experimentName)
	}
	
	public def String stateless_applyEglScript(String modelName,
												String modelURI,
												String metaModelFilePath, 
												String eglScriptPath, 
												String experimentName){
		
//		var inputQpnModel = createEmfModel_usingUri("QPME", modelURI, "models/qpme.ecore", true, false);
		var inputQpnModel = createEmfModel_usingUri(modelName, modelURI, metaModelFilePath, true, false);
		var eglModule = new EglTemplateFactoryModuleAdapter(new EglFileGeneratingTemplateFactory());
		var qpme_file_contents = prepareAndExecute(eglModule, eglScriptPath, newLinkedList(inputQpnModel), newLinkedList())
		return qpme_file_contents as String
	}
	
	/*
	 * Walks over qpmeModels list and generates text representations of the qpme models
	 */
	protected def generateQpeFiles(){
		
		for(qpnURI : qpmeModels){
			
			var experimentName = (qpnURI as ExperimentModel).experimentName
			println("EXPERIMENT NAME!!!!!!! " + experimentName)
			var modelURI = (qpnURI as ExperimentModel).modelURI
			
			var String contents = modelToTextQpn(modelURI, experimentName)
			
			var outFileName = experimentName+".qpe"
			var outputQpnURI = pwdURI+qpmeModelsDirRelative+outFileName
			
			var unixFilePath = outputQpnURI.replace("file:/","/")
			saveOutput(contents, unixFilePath)
			
			println("Saving model "+outFileName+"\n to file: "+unixFilePath)
			
			qpmeFiles.add(new ExperimentModel(experimentName, unixFilePath))
		}
	}
	
	/*
	 * Runs a eol script over the generated dni models. Gets the models form the dniModels list
	 */
	protected def checkDniModels(){
		for(dniURI : dniModels){
			var experimentName = (dniURI as ExperimentModel).experimentName
			var modelURI = (dniURI as ExperimentModel).modelURI
			
			var emfModel = createEmfModel_usingUri("DNI", modelURI, "models/dnimm3.ecore", true, false);
			prepareAndExecute(new EolModule(), "eol/simutools_check.eol", newLinkedList(emfModel), newLinkedList);
		}
	}
	
	/*
	 * Takes base_dni_file and generates its variations using the parameters and the eol script 
	 * The models are stored in the dniModels list
	 */
	protected def generateDniModel(String base_dni_file, List<Variable> eolParameters, int index, String experimentName){
		
		println("###### USING BASE MODEL: "+base_dni_file+"######")
		var ExperimentModel pairExpNameDniModel;
		pairExpNameDniModel = stateless_applyEolStriptToBaseModel(base_dni_file, "eol/simutools_experiment3.eol", eolParameters, index, experimentName);
		dniModels.add(pairExpNameDniModel)		
	}
	
	protected def generateDniModel(String dni_file, String experimentName) {
		println("###### Transforming Mode: "+dni_file+"#####")
		var ExperimentModel m
		var String resultURI
		if(experimentName.isEmpty) 
			resultURI = pwdURI+dniModelsDirRelative+"dni_.dnimm3"
		else
			resultURI = pwdURI+dniModelsDirRelative+experimentName+".dnimm3"
		
		var dni_model = createEmfModel("DNI", dni_file, "metamodels/dnimm3.ecore", true, true);
		dni_model.store(resultURI)
		m = new ExperimentModel(experimentName, resultURI)
		dniModels.add(m)
	}
	
	public def ExperimentModel stateless_applyEolStriptToBaseModel(String base_dni_file, String eolScriptPath, List<Variable> eolParameters, int index, String experimentName){
		
	 
		var base_dni_model = createEmfModel("DNI", base_dni_file, "metamodels/dnimm3.ecore", true, true);
		prepareAndExecute(new EolModule(), eolScriptPath, newLinkedList(base_dni_model), eolParameters);
		
		var String resultURI
		if(experimentName.isEmpty) 
			resultURI = pwdURI+dniModelsDirRelative+"dni_"+index+".dnimm3"
		else
			resultURI = pwdURI+dniModelsDirRelative+experimentName+".dnimm3"
			
		base_dni_model.store(resultURI)
		
		var ExperimentModel expName2dniModel = new ExperimentModel(experimentName, resultURI)
		return expName2dniModel
	}
	
	/*
	 * Processes the dniModels list and generate a qpn representation for each. Qpn models are saved to the qpmeModels list
	 */
	protected def transformAll(){
		for(dniURI : dniModels){
			
			var experimentName = (dniURI as ExperimentModel).experimentName
			var modelURI = (dniURI as ExperimentModel).modelURI
			
			var qpmeResultURI = transformDniToQpn(modelURI, experimentName)
			qpmeModels.add(new ExperimentModel(experimentName, qpmeResultURI))
		}
	}
	
	/*
	 * Processes a single dni model and returns uri to the qpn representation
	 */
	private def String transformDniToQpn(String sourceURI, String experimentName){
		
			
		var List<Variable> origParameterList = epsilonTransformationParameters.get(experimentName)
		
		var outFileName = experimentName+".qpmemodel"
		var destinationModelURI = pwdURI+qpmeModelsDirRelative+outFileName
		
		var String outputQpnURI = stateless_applyEtlStript("DNI", sourceURI, "metamodels/dnimm3.ecore", 
													"QPME", destinationModelURI, "metamodels/qpme.ecore",
													"etl/standalone-DNI-QPN.etl", origParameterList, experimentName)
		return outputQpnURI
	}
	
	public def String stateless_applyEtlStript(
		String sourceModelName,
		String sourceModelURI,
		String sourceMetaModelURI,
		String destinationModelName,
		String destinationModelURI,
		String destinationMetaModelURI,  
		String etlScriptPath, 
		List<Variable> eolParameters, 
		String experimentName)
	{
		
				
		var inputDniModel = createEmfModel_usingUri(sourceModelName, sourceModelURI, sourceMetaModelURI, true, false);
		var outputQpnModel = createEmfModel_usingUri(destinationModelName, destinationModelURI, destinationMetaModelURI, false, true);
				
		prepareAndExecute(new EtlModule(), etlScriptPath, newLinkedList(inputDniModel, outputQpnModel), eolParameters);
		outputQpnModel.store(destinationModelURI)
		return destinationModelURI			
	}
	
	protected def cleanup(){
		
		var resultsDir = pwdUnix+qpmeResultsDirRelative
		var List<String> existingResultsFileList = getFileList(resultsDir, ".results.xml", "")
		var List<String> flatHQPNfiles = getFileList(pwdUnix, ".xml", "SimQPN_FlatHQPN_")
		
		var List<String> toDelete = newLinkedList()
		toDelete.addAll(existingResultsFileList)
		toDelete.addAll(flatHQPNfiles)
		
		for(fileToDelete : existingResultsFileList ){
			var boolean deleted = deleteFile(fileToDelete)
			if(deleted)
				println("Deleted file "+fileToDelete)
			else
				println("Could not delete file "+fileToDelete)
		}
	}
	
	
	
	
	
	
	
	/*
	 * Executes Epsilon code over models 
	 */
	private def Object prepareAndExecute(IEolModule module, String source, List<IModel> models, List<Variable> parameters) throws Exception {

		var resURI = getFileURI(source);
    	module.parse(resURI);
    
    	if (module.getParseProblems().size() > 0) {
			System.err.println("Parse errors occured...");
			for (ParseProblem problem : module.getParseProblems()) {
        		System.err.println(problem.toString());
      		}
      		return null;
    	}
    
    	for (IModel model : models) {
			module.getContext().getModelRepository().addModel(model);
    	}
    
    	for (Variable parameter : parameters) {
      		module.getContext().getFrameStack().put(parameter);
    	}
    
    
    	var result = execute(module);
    
    
    	for (IModel model : models) {
    		module.getContext().getModelRepository().removeModel(model)	
    	}
    	//module.getContext().getModelRepository().dispose(); //this prohibits executing multiple transformations in one run
    	// but commented out does not delete the models
    	return result;
	}
	
	protected def Object execute(IEolModule module) throws EolRuntimeException {
		return module.execute();
  	}
  	
  	/*
  	 * Creates emf model object if a file addressed over uri exists. Uses file-based META-model
  	 */
	protected def EmfModel createEmfModel(String name, String model, 
		String metamodel, boolean readOnLoad, boolean storeOnDisposal)  throws EolModelLoadingException, URISyntaxException {
          	
		//println("createEmfModel name "+name+" model "+model)
          	
    	var EmfModel emfModel = new EmfModel();
    	var StringProperties properties = new StringProperties();
    	properties.put(EmfModel.PROPERTY_NAME, name);
    	properties.put(EmfModel.PROPERTY_FILE_BASED_METAMODEL_URI, getFileURI(metamodel).toString());
    	properties.put(EmfModel.PROPERTY_MODEL_URI, getFileURI(model).toString());
    	properties.put(EmfModel.PROPERTY_READONLOAD, readOnLoad + "");
    	properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, storeOnDisposal + "");
    	//emfModel.load(properties, (IRelativePathResolver) null);
    	emfModel.load(properties, null as IRelativePathResolver);
    	return emfModel;
  	}
  	
  	/*
  	 * Creates emf model object if a file addressed over uri does not exist
  	 */  	
	protected def EmfModel createEmfModel_usingUri(String name, String model, 
		String metamodel, boolean readOnLoad, boolean storeOnDisposal)  throws EolModelLoadingException, URISyntaxException {
          	
		println("createEmfModel_usingUri name "+name+" model "+model+", metamodel "+metamodel)
          	
    	var EmfModel emfModel = new EmfModel();
    	var StringProperties properties = new StringProperties();
    	properties.put(EmfModel.PROPERTY_NAME, name);
    	properties.put(EmfModel.PROPERTY_FILE_BASED_METAMODEL_URI, getFileURI(metamodel).toString());
    	properties.put(EmfModel.PROPERTY_MODEL_URI, model);
    	properties.put(EmfModel.PROPERTY_READONLOAD, readOnLoad + "");
    	properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, storeOnDisposal + "");
    	//emfModel.load(properties, (IRelativePathResolver) null);
    	emfModel.load(properties, null as IRelativePathResolver);
    	
    	
    	
    	return emfModel;
  	}

	 /*
  	 * The same as createEmfModel but allows addressing of META-model using uri
  	 */
	protected def EmfModel createEmfModelByURI(String name, String model, 
  					String metamodel, boolean readOnLoad, boolean storeOnDisposal) 
          			throws EolModelLoadingException, URISyntaxException {
		var EmfModel emfModel = new EmfModel();
		var StringProperties properties = new StringProperties();
		properties.put(EmfModel.PROPERTY_NAME, name);
		properties.put(EmfModel.PROPERTY_METAMODEL_URI, metamodel);
		properties.put(EmfModel.PROPERTY_MODEL_URI, getFileURI(model).toString());
		properties.put(EmfModel.PROPERTY_READONLOAD, readOnLoad + "");
		properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, storeOnDisposal + "");
		emfModel.load(properties, null as IRelativePathResolver);
		return emfModel;
  	}
  
	def private URI getFileURI(String fileName) throws URISyntaxException {
		//var URI binUri = typeof(EpsilonStandaloneExample).getResource(fileName).toURI()
		
		var res = DNIRunner.getResource(fileName)
		//println("getFileURI(String fileName) "+fileName)
		var URI binUri = res.toURI()
		var URI uri = null
	
		if (binUri.toString().indexOf("bin") > -1) {
	  		uri = new URI(binUri.toString().replaceAll("bin", "src"));
	    	}
	    	else {
	      		uri = binUri;
	    	}
	    	
	    	return uri;
	}
	  
	def saveOutput(String txt, String fname){
		var File file = new File(fname);
      	file.createNewFile();
      	var FileWriter writer = new FileWriter(file); 
      	writer.write(txt); 
      	writer.flush();
      	writer.close();
	}
	
}
