package epsilonStandalone;

import com.google.common.base.Objects;
import epsilonStandalone.ExperimentModel;
import epsilonStandalone.SimQpnResultsParser;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.eclipse.epsilon.common.parse.problem.ParseProblem;
import org.eclipse.epsilon.common.util.StringProperties;
import org.eclipse.epsilon.egl.EglFileGeneratingTemplateFactory;
import org.eclipse.epsilon.egl.EglTemplateFactoryModuleAdapter;
import org.eclipse.epsilon.emc.emf.EmfModel;
import org.eclipse.epsilon.eol.EolModule;
import org.eclipse.epsilon.eol.IEolExecutableModule;
import org.eclipse.epsilon.eol.IEolModule;
import org.eclipse.epsilon.eol.exceptions.EolRuntimeException;
import org.eclipse.epsilon.eol.exceptions.models.EolModelLoadingException;
import org.eclipse.epsilon.eol.execute.context.FrameStack;
import org.eclipse.epsilon.eol.execute.context.IEolContext;
import org.eclipse.epsilon.eol.execute.context.Variable;
import org.eclipse.epsilon.eol.models.IModel;
import org.eclipse.epsilon.eol.models.IRelativePathResolver;
import org.eclipse.epsilon.eol.models.ModelRepository;
import org.eclipse.epsilon.eol.types.EolPrimitiveType;
import org.eclipse.epsilon.etl.EtlModule;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IterableExtensions;

@SuppressWarnings("all")
public class DNIRunner {
  protected String simqpn = "bash /home/mk/workspaces/workspace_DNI_neo/DNI3/DNIinstances/simutools2016/results/runSimQPN.sh -r simutools2016 ";
  
  protected boolean simulationSilent = true;
  
  protected int kiloMultipler = 1000;
  
  protected String pwdUnix = (System.getProperty("user.dir") + "/");
  
  protected String pwdURI = ("file:" + this.pwdUnix);
  
  protected String dniModelsDirRelative = "src/epsilonStandalone/models/dni/";
  
  protected String tmpModelsDirRelative = "src/epsilonStandalone/models/tmp/";
  
  protected String qpmeModelsDirRelative = "src/epsilonStandalone/models/qpme/";
  
  protected String qpmeResultsDirRelative = "src/epsilonStandalone/models/qpmeResults/";
  
  protected String dni_base_model_file = "models/scenario0A.model";
  
  protected final LinkedList<Object> dniModels = CollectionLiterals.<Object>newLinkedList();
  
  protected final LinkedList<Object> qpmeModels = CollectionLiterals.<Object>newLinkedList();
  
  protected final LinkedList<Object> qpmeFiles = CollectionLiterals.<Object>newLinkedList();
  
  protected final LinkedList<Object> qpmeSimulationResultFiles = CollectionLiterals.<Object>newLinkedList();
  
  protected final LinkedList<Object> qpmeSimulationProcessedResults = CollectionLiterals.<Object>newLinkedList();
  
  protected SimQpnResultsParser simQpnParser = new SimQpnResultsParser();
  
  protected HashMap<String, List<Variable>> epsilonTransformationParameters = new HashMap<String, List<Variable>>();
  
  protected String currentTransformationFile = "etl/standalone-DNI-QPN-new.etl";
  
  public static void main(final String[] args) {
    DNIRunner _dNIRunner = new DNIRunner();
    _dNIRunner.start_single();
  }
  
  public void start_single() {
    LinkedList<String> _newLinkedList = CollectionLiterals.<String>newLinkedList("SW35");
    this.simQpnParser.interestingPlaces = _newLinkedList;
    LinkedList<String> _newLinkedList_1 = CollectionLiterals.<String>newLinkedList("Traffic");
    this.simQpnParser.interestingColors = _newLinkedList_1;
    LinkedList<String> _newLinkedList_2 = CollectionLiterals.<String>newLinkedList("arrivThrPut");
    this.simQpnParser.interestingMetrics = _newLinkedList_2;
    this.epsilonTransformationParameters.clear();
    String expNamePrefix = "AA_Debug_run";
    String experimentName = expNamePrefix;
    ExperimentModel pairListExpNameDniModel = null;
    LinkedList<Variable> _newLinkedList_3 = CollectionLiterals.<Variable>newLinkedList();
    ExperimentModel _stateless_applyEolStriptToBaseModel = this.stateless_applyEolStriptToBaseModel(this.dni_base_model_file, "eol/simutools_check.eol", _newLinkedList_3, 0, experimentName);
    pairListExpNameDniModel = _stateless_applyEolStriptToBaseModel;
    String dniModelURI = pairListExpNameDniModel.getModelURI();
    Variable _variable = new Variable("param_goOverSdnSW", Boolean.valueOf(true), EolPrimitiveType.Boolean);
    List<Variable> paramList = CollectionLiterals.<Variable>newLinkedList(_variable);
    String outFileName = (experimentName + ".qpmemodel");
    String destinationModelURI = ((this.pwdURI + this.qpmeModelsDirRelative) + outFileName);
    InputOutput.<String>println(("Will store transformation result in: " + destinationModelURI));
    destinationModelURI = "file:models/qpme/demo.qpmemodel";
    String outputQpnURI = this.stateless_applyEtlStript("DNI", dniModelURI, "metamodels/dnimm3.ecore", 
      "QPME", destinationModelURI, "metamodels/qpme.ecore", 
      this.currentTransformationFile, paramList, experimentName);
    String qpeFileContents = this.stateless_applyEglScript("QPME", outputQpnURI, "metamodels/qpme.ecore", "egl/dni2qpme.egl", experimentName);
    String outputQpeFilePath = (((this.pwdUnix + this.qpmeModelsDirRelative) + experimentName) + ".qpe");
    this.saveOutput(qpeFileContents, outputQpeFilePath);
    InputOutput.<String>println(("Your .qpe file is here: " + outputQpeFilePath));
    this.cleanup();
  }
  
  private boolean deleteFile(final String path) {
    File f = null;
    File _file = new File(path);
    f = _file;
    return f.delete();
  }
  
  public static String getCurrentTimeStamp() {
    SimpleDateFormat sdfDate = new SimpleDateFormat("yyy.MM.dd.HH.mm.ss");
    Date now = new Date();
    String strDate = sdfDate.format(now);
    return strDate;
  }
  
  private List<String> getFileList(final String directory, final String endsWith, final String contains) {
    List<String> files = CollectionLiterals.<String>newLinkedList();
    File folder = new File(directory);
    File[] listOfFiles = folder.listFiles();
    for (final File f : listOfFiles) {
      {
        File file = ((File) f);
        boolean _isFile = file.isFile();
        if (_isFile) {
          String fname = file.getName();
          if (((fname.endsWith(endsWith) && (!endsWith.isEmpty())) && contains.isEmpty())) {
            String path = ((directory + "/") + fname);
            files.add(path);
          } else {
            if (((fname.contains(contains) && (!contains.isEmpty())) && (!contains.isEmpty()))) {
              String path_1 = ((directory + "/") + fname);
              files.add(path_1);
            } else {
              if ((fname.contains(contains) && fname.endsWith(endsWith))) {
                String path_2 = ((directory + "/") + fname);
                files.add(path_2);
              }
            }
          }
        }
      }
    }
    return files;
  }
  
  protected void parseAllResults(final String finalFileName) {
    String finalResult = "";
    for (final Object resultFile : this.qpmeSimulationResultFiles) {
      {
        String experimentName = ((ExperimentModel) resultFile).getExperimentName();
        String qpeResultFile = ((ExperimentModel) resultFile).getModelURI();
        String result = this.parseResult(qpeResultFile, experimentName);
        String line = ((experimentName + " # ") + result);
        String _finalResult = finalResult;
        finalResult = (_finalResult + line);
        InputOutput.<String>println(line);
        this.qpmeSimulationProcessedResults.add(result);
      }
    }
    this.saveOutput(finalResult, ((this.pwdUnix + this.qpmeResultsDirRelative) + finalFileName));
  }
  
  public String parseResult(final String qpeResultFile, final String experimentName) {
    try {
      String _xblockexpression = null;
      {
        String toReturn = "";
        File _file = new File(qpeResultFile);
        boolean _isFile = _file.isFile();
        boolean _equals = (_isFile == false);
        if (_equals) {
          return " File does not exist.";
        }
        HashMap<String, HashMap<String, Double>> summary = new HashMap<String, HashMap<String, Double>>();
        HashMap<String, HashMap<String, LinkedHashMap<String, Double>>> map = this.simQpnParser.parse(qpeResultFile);
        Set<Map.Entry<String, HashMap<String, LinkedHashMap<String, Double>>>> _entrySet = map.entrySet();
        Iterator<Map.Entry<String, HashMap<String, LinkedHashMap<String, Double>>>> i1 = _entrySet.iterator();
        while (i1.hasNext()) {
          {
            Map.Entry<String, HashMap<String, LinkedHashMap<String, Double>>> _next = i1.next();
            Map.Entry<String, HashMap<String, LinkedHashMap<String, Double>>> me = ((Map.Entry<String, HashMap<String, LinkedHashMap<String, Double>>>) _next);
            String _key = me.getKey();
            String placeName = ((String) _key);
            HashMap<String, LinkedHashMap<String, Double>> _value = me.getValue();
            HashMap<String, LinkedHashMap<String, Double>> colorMetricValueHM = ((HashMap<String, LinkedHashMap<String, Double>>) _value);
            Set<Map.Entry<String, LinkedHashMap<String, Double>>> _entrySet_1 = colorMetricValueHM.entrySet();
            Iterator<Map.Entry<String, LinkedHashMap<String, Double>>> i2 = _entrySet_1.iterator();
            while (i2.hasNext()) {
              {
                Map.Entry<String, LinkedHashMap<String, Double>> _next_1 = i2.next();
                Map.Entry<String, LinkedHashMap<String, Double>> me2 = ((Map.Entry<String, LinkedHashMap<String, Double>>) _next_1);
                String _key_1 = me2.getKey();
                String colorName = ((String) _key_1);
                LinkedHashMap<String, Double> _value_1 = me2.getValue();
                LinkedHashMap<String, Double> metricValueHM = ((LinkedHashMap<String, Double>) _value_1);
                Set<Map.Entry<String, Double>> _entrySet_2 = metricValueHM.entrySet();
                Iterator<Map.Entry<String, Double>> i3 = _entrySet_2.iterator();
                while (i3.hasNext()) {
                  {
                    Map.Entry<String, Double> _next_2 = i3.next();
                    Map.Entry<String, Double> me3 = ((Map.Entry<String, Double>) _next_2);
                    String _key_2 = me3.getKey();
                    String metricName = ((String) _key_2);
                    Double _value_2 = me3.getValue();
                    double value = (((Double) _value_2)).doubleValue();
                    double tokensPerSec = value;
                    double bytes = this.simQpnParser.parseColorMessageSize(colorName, this.kiloMultipler);
                    double bytesPerSec = (tokensPerSec * bytes);
                    double bitsPerSec = (bytesPerSec * 8);
                    double MbitsPerSec = ((bitsPerSec / 1000) / 1000);
                    DecimalFormat _decimalFormat = new DecimalFormat("#.##");
                    String MbitsPerSecTxt = _decimalFormat.format(MbitsPerSec);
                    boolean _containsKey = summary.containsKey(placeName);
                    boolean _not = (!_containsKey);
                    if (_not) {
                      HashMap<String, Double> _hashMap = new HashMap<String, Double>();
                      summary.put(placeName, _hashMap);
                    }
                    HashMap<String, Double> _get = summary.get(placeName);
                    _get.put(colorName, Double.valueOf(MbitsPerSec));
                  }
                }
              }
            }
          }
        }
        Set<String> _keySet = summary.keySet();
        List<String> keyList = IterableExtensions.<String>toList(_keySet);
        Collections.<String>sort(keyList);
        for (final String k : keyList) {
          {
            String _return = toReturn;
            String _friendlyName = this.simQpnParser.getFriendlyName(k);
            String _plus = (_friendlyName + " --- ");
            toReturn = (_return + _plus);
            HashMap<String, Double> _get = summary.get(k);
            Set<String> _keySet_1 = _get.keySet();
            List<String> innerList = IterableExtensions.<String>toList(_keySet_1);
            Collections.<String>sort(innerList);
            double sum = 0.0;
            for (final String kk : innerList) {
              double _sum = sum;
              HashMap<String, Double> _get_1 = summary.get(k);
              Double _get_2 = _get_1.get(kk);
              sum = (_sum + (_get_2).doubleValue());
            }
            String _return_1 = toReturn;
            String _plus_1 = (Double.valueOf(sum) + " Mbps\n");
            toReturn = (_return_1 + _plus_1);
          }
        }
        _xblockexpression = toReturn;
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  protected void simulateAll() {
    int i = 0;
    for (final Object qpmeFile : this.qpmeFiles) {
      {
        int _i = i;
        i = (_i + 1);
        int _size = this.qpmeFiles.size();
        String _plus = ((("Simulating run #" + Integer.valueOf(i)) + " of ") + Integer.valueOf(_size));
        InputOutput.<String>println(_plus);
        String experimentName = ((ExperimentModel) qpmeFile).getExperimentName();
        String qpeFilePath = ((ExperimentModel) qpmeFile).getModelURI();
        String resultsFilePath = this.simulateSimQPN(qpeFilePath, experimentName);
        InputOutput.<String>println((((("Adding " + experimentName) + " with path ") + qpeFilePath) + " to qpeSimulationResultFiles"));
        ExperimentModel _experimentModel = new ExperimentModel(experimentName, resultsFilePath);
        this.qpmeSimulationResultFiles.add(_experimentModel);
      }
    }
  }
  
  private String simulateSimQPN(final String qpeFile, final String experimentName) {
    try {
      String simulation_output_file = (("/home/mk/workspaces/workspace_DNI_neo/DNI3/DNIinstances/" + experimentName) + ".results.xml");
      String _replace = simulation_output_file.replace("file:/", "/");
      simulation_output_file = _replace;
      String output_simulation = (" -v /dev/null -o " + simulation_output_file);
      String command = ((this.simqpn + qpeFile) + output_simulation);
      InputOutput.<String>println(("COMMAND: " + command));
      Runtime _runtime = Runtime.getRuntime();
      Process p = _runtime.exec(command);
      InputStream _inputStream = p.getInputStream();
      InputStreamReader _inputStreamReader = new InputStreamReader(_inputStream);
      BufferedReader stdInput = new BufferedReader(_inputStreamReader);
      InputStream _errorStream = p.getErrorStream();
      InputStreamReader _inputStreamReader_1 = new InputStreamReader(_errorStream);
      BufferedReader stdError = new BufferedReader(_inputStreamReader_1);
      String s = stdInput.readLine();
      while ((!Objects.equal(s, null))) {
        {
          if ((!this.simulationSilent)) {
            InputOutput.<String>println(s);
          }
          String _readLine = stdInput.readLine();
          s = _readLine;
        }
      }
      String se = stdError.readLine();
      while ((!Objects.equal(se, null))) {
        {
          if ((!this.simulationSilent)) {
            InputOutput.<String>println(se);
          }
          String _readLine = stdError.readLine();
          se = _readLine;
        }
      }
      return simulation_output_file;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  /**
   * Takes qpme model and returns the contents of the qpe file
   */
  private String modelToTextQpn(final String sourceURI, final String experimentName) {
    return this.stateless_applyEglScript("QPME", sourceURI, "models/qpme.ecore", "egl/dni2qpme.egl", experimentName);
  }
  
  public String stateless_applyEglScript(final String modelName, final String modelURI, final String metaModelFilePath, final String eglScriptPath, final String experimentName) {
    try {
      EmfModel inputQpnModel = this.createEmfModel_usingUri(modelName, modelURI, metaModelFilePath, true, false);
      EglFileGeneratingTemplateFactory _eglFileGeneratingTemplateFactory = new EglFileGeneratingTemplateFactory();
      EglTemplateFactoryModuleAdapter eglModule = new EglTemplateFactoryModuleAdapter(_eglFileGeneratingTemplateFactory);
      LinkedList<IModel> _newLinkedList = CollectionLiterals.<IModel>newLinkedList(inputQpnModel);
      LinkedList<Variable> _newLinkedList_1 = CollectionLiterals.<Variable>newLinkedList();
      Object qpme_file_contents = this.prepareAndExecute(eglModule, eglScriptPath, _newLinkedList, _newLinkedList_1);
      return ((String) qpme_file_contents);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  /**
   * Walks over qpmeModels list and generates text representations of the qpme models
   */
  protected void generateQpeFiles() {
    for (final Object qpnURI : this.qpmeModels) {
      {
        String experimentName = ((ExperimentModel) qpnURI).getExperimentName();
        InputOutput.<String>println(("EXPERIMENT NAME!!!!!!! " + experimentName));
        String modelURI = ((ExperimentModel) qpnURI).getModelURI();
        String contents = this.modelToTextQpn(modelURI, experimentName);
        String outFileName = (experimentName + ".qpe");
        String outputQpnURI = ((this.pwdURI + this.qpmeModelsDirRelative) + outFileName);
        String unixFilePath = outputQpnURI.replace("file:/", "/");
        this.saveOutput(contents, unixFilePath);
        InputOutput.<String>println(((("Saving model " + outFileName) + "\n to file: ") + unixFilePath));
        ExperimentModel _experimentModel = new ExperimentModel(experimentName, unixFilePath);
        this.qpmeFiles.add(_experimentModel);
      }
    }
  }
  
  /**
   * Runs a eol script over the generated dni models. Gets the models form the dniModels list
   */
  protected void checkDniModels() {
    try {
      for (final Object dniURI : this.dniModels) {
        {
          String experimentName = ((ExperimentModel) dniURI).getExperimentName();
          String modelURI = ((ExperimentModel) dniURI).getModelURI();
          EmfModel emfModel = this.createEmfModel_usingUri("DNI", modelURI, "models/dnimm3.ecore", true, false);
          EolModule _eolModule = new EolModule();
          LinkedList<IModel> _newLinkedList = CollectionLiterals.<IModel>newLinkedList(emfModel);
          LinkedList<Variable> _newLinkedList_1 = CollectionLiterals.<Variable>newLinkedList();
          this.prepareAndExecute(_eolModule, "eol/simutools_check.eol", _newLinkedList, _newLinkedList_1);
        }
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  /**
   * Takes base_dni_file and generates its variations using the parameters and the eol script
   * The models are stored in the dniModels list
   */
  protected boolean generateDniModel(final String base_dni_file, final List<Variable> eolParameters, final int index, final String experimentName) {
    boolean _xblockexpression = false;
    {
      InputOutput.<String>println((("###### USING BASE MODEL: " + base_dni_file) + "######"));
      ExperimentModel pairExpNameDniModel = null;
      ExperimentModel _stateless_applyEolStriptToBaseModel = this.stateless_applyEolStriptToBaseModel(base_dni_file, "eol/simutools_experiment3.eol", eolParameters, index, experimentName);
      pairExpNameDniModel = _stateless_applyEolStriptToBaseModel;
      _xblockexpression = this.dniModels.add(pairExpNameDniModel);
    }
    return _xblockexpression;
  }
  
  protected boolean generateDniModel(final String dni_file, final String experimentName) {
    try {
      boolean _xblockexpression = false;
      {
        InputOutput.<String>println((("###### Transforming Mode: " + dni_file) + "#####"));
        ExperimentModel m = null;
        String resultURI = null;
        boolean _isEmpty = experimentName.isEmpty();
        if (_isEmpty) {
          resultURI = ((this.pwdURI + this.dniModelsDirRelative) + "dni_.dnimm3");
        } else {
          resultURI = (((this.pwdURI + this.dniModelsDirRelative) + experimentName) + ".dnimm3");
        }
        EmfModel dni_model = this.createEmfModel("DNI", dni_file, "metamodels/dnimm3.ecore", true, true);
        dni_model.store(resultURI);
        ExperimentModel _experimentModel = new ExperimentModel(experimentName, resultURI);
        m = _experimentModel;
        _xblockexpression = this.dniModels.add(m);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public ExperimentModel stateless_applyEolStriptToBaseModel(final String base_dni_file, final String eolScriptPath, final List<Variable> eolParameters, final int index, final String experimentName) {
    try {
      EmfModel base_dni_model = this.createEmfModel("DNI", base_dni_file, "metamodels/dnimm3.ecore", true, true);
      EolModule _eolModule = new EolModule();
      LinkedList<IModel> _newLinkedList = CollectionLiterals.<IModel>newLinkedList(base_dni_model);
      this.prepareAndExecute(_eolModule, eolScriptPath, _newLinkedList, eolParameters);
      String resultURI = null;
      boolean _isEmpty = experimentName.isEmpty();
      if (_isEmpty) {
        resultURI = ((((this.pwdURI + this.dniModelsDirRelative) + "dni_") + Integer.valueOf(index)) + ".dnimm3");
      } else {
        resultURI = (((this.pwdURI + this.dniModelsDirRelative) + experimentName) + ".dnimm3");
      }
      base_dni_model.store(resultURI);
      ExperimentModel expName2dniModel = new ExperimentModel(experimentName, resultURI);
      return expName2dniModel;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  /**
   * Processes the dniModels list and generate a qpn representation for each. Qpn models are saved to the qpmeModels list
   */
  protected void transformAll() {
    for (final Object dniURI : this.dniModels) {
      {
        String experimentName = ((ExperimentModel) dniURI).getExperimentName();
        String modelURI = ((ExperimentModel) dniURI).getModelURI();
        String qpmeResultURI = this.transformDniToQpn(modelURI, experimentName);
        ExperimentModel _experimentModel = new ExperimentModel(experimentName, qpmeResultURI);
        this.qpmeModels.add(_experimentModel);
      }
    }
  }
  
  /**
   * Processes a single dni model and returns uri to the qpn representation
   */
  private String transformDniToQpn(final String sourceURI, final String experimentName) {
    List<Variable> origParameterList = this.epsilonTransformationParameters.get(experimentName);
    String outFileName = (experimentName + ".qpmemodel");
    String destinationModelURI = ((this.pwdURI + this.qpmeModelsDirRelative) + outFileName);
    String outputQpnURI = this.stateless_applyEtlStript("DNI", sourceURI, "metamodels/dnimm3.ecore", 
      "QPME", destinationModelURI, "metamodels/qpme.ecore", 
      "etl/standalone-DNI-QPN.etl", origParameterList, experimentName);
    return outputQpnURI;
  }
  
  public String stateless_applyEtlStript(final String sourceModelName, final String sourceModelURI, final String sourceMetaModelURI, final String destinationModelName, final String destinationModelURI, final String destinationMetaModelURI, final String etlScriptPath, final List<Variable> eolParameters, final String experimentName) {
    try {
      EmfModel inputDniModel = this.createEmfModel_usingUri(sourceModelName, sourceModelURI, sourceMetaModelURI, true, false);
      EmfModel outputQpnModel = this.createEmfModel_usingUri(destinationModelName, destinationModelURI, destinationMetaModelURI, false, true);
      EtlModule _etlModule = new EtlModule();
      LinkedList<IModel> _newLinkedList = CollectionLiterals.<IModel>newLinkedList(inputDniModel, outputQpnModel);
      this.prepareAndExecute(_etlModule, etlScriptPath, _newLinkedList, eolParameters);
      outputQpnModel.store(destinationModelURI);
      return destinationModelURI;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  protected void cleanup() {
    String resultsDir = (this.pwdUnix + this.qpmeResultsDirRelative);
    List<String> existingResultsFileList = this.getFileList(resultsDir, ".results.xml", "");
    List<String> flatHQPNfiles = this.getFileList(this.pwdUnix, ".xml", "SimQPN_FlatHQPN_");
    List<String> toDelete = CollectionLiterals.<String>newLinkedList();
    toDelete.addAll(existingResultsFileList);
    toDelete.addAll(flatHQPNfiles);
    for (final String fileToDelete : existingResultsFileList) {
      {
        boolean deleted = this.deleteFile(fileToDelete);
        if (deleted) {
          InputOutput.<String>println(("Deleted file " + fileToDelete));
        } else {
          InputOutput.<String>println(("Could not delete file " + fileToDelete));
        }
      }
    }
  }
  
  /**
   * Executes Epsilon code over models
   */
  private Object prepareAndExecute(final IEolExecutableModule module, final String source, final List<IModel> models, final List<Variable> parameters) throws Exception {
    URI resURI = this.getFileURI(source);
    module.parse(resURI);
    List<ParseProblem> _parseProblems = module.getParseProblems();
    int _size = _parseProblems.size();
    boolean _greaterThan = (_size > 0);
    if (_greaterThan) {
      System.err.println("Parse errors occured...");
      List<ParseProblem> _parseProblems_1 = module.getParseProblems();
      for (final ParseProblem problem : _parseProblems_1) {
        String _string = problem.toString();
        System.err.println(_string);
      }
      return null;
    }
    for (final IModel model : models) {
      IEolContext _context = module.getContext();
      ModelRepository _modelRepository = _context.getModelRepository();
      _modelRepository.addModel(model);
    }
    for (final Variable parameter : parameters) {
      IEolContext _context_1 = module.getContext();
      FrameStack _frameStack = _context_1.getFrameStack();
      _frameStack.put(parameter);
    }
    Object result = this.execute(module);
    for (final IModel model_1 : models) {
      IEolContext _context_2 = module.getContext();
      ModelRepository _modelRepository_1 = _context_2.getModelRepository();
      _modelRepository_1.removeModel(model_1);
    }
    return result;
  }
  
  protected Object execute(final IEolExecutableModule module) throws EolRuntimeException {
    return module.execute();
  }
  
  /**
   * Creates emf model object if a file addressed over uri exists. Uses file-based META-model
   */
  protected EmfModel createEmfModel(final String name, final String model, final String metamodel, final boolean readOnLoad, final boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
    EmfModel emfModel = new EmfModel();
    StringProperties properties = new StringProperties();
    properties.put(EmfModel.PROPERTY_NAME, name);
    URI _fileURI = this.getFileURI(metamodel);
    String _string = _fileURI.toString();
    properties.put(EmfModel.PROPERTY_FILE_BASED_METAMODEL_URI, _string);
    URI _fileURI_1 = this.getFileURI(model);
    String _string_1 = _fileURI_1.toString();
    properties.put(EmfModel.PROPERTY_MODEL_URI, _string_1);
    String _plus = (Boolean.valueOf(readOnLoad) + "");
    properties.put(EmfModel.PROPERTY_READONLOAD, _plus);
    String _plus_1 = (Boolean.valueOf(storeOnDisposal) + "");
    properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, _plus_1);
    emfModel.load(properties, ((IRelativePathResolver) null));
    return emfModel;
  }
  
  /**
   * Creates emf model object if a file addressed over uri does not exist
   */
  protected EmfModel createEmfModel_usingUri(final String name, final String model, final String metamodel, final boolean readOnLoad, final boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
    InputOutput.<String>println(((((("createEmfModel_usingUri name " + name) + " model ") + model) + ", metamodel ") + metamodel));
    EmfModel emfModel = new EmfModel();
    StringProperties properties = new StringProperties();
    properties.put(EmfModel.PROPERTY_NAME, name);
    URI _fileURI = this.getFileURI(metamodel);
    String _string = _fileURI.toString();
    properties.put(EmfModel.PROPERTY_FILE_BASED_METAMODEL_URI, _string);
    properties.put(EmfModel.PROPERTY_MODEL_URI, model);
    String _plus = (Boolean.valueOf(readOnLoad) + "");
    properties.put(EmfModel.PROPERTY_READONLOAD, _plus);
    String _plus_1 = (Boolean.valueOf(storeOnDisposal) + "");
    properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, _plus_1);
    emfModel.load(properties, ((IRelativePathResolver) null));
    return emfModel;
  }
  
  /**
   * The same as createEmfModel but allows addressing of META-model using uri
   */
  protected EmfModel createEmfModelByURI(final String name, final String model, final String metamodel, final boolean readOnLoad, final boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
    EmfModel emfModel = new EmfModel();
    StringProperties properties = new StringProperties();
    properties.put(EmfModel.PROPERTY_NAME, name);
    properties.put(EmfModel.PROPERTY_METAMODEL_URI, metamodel);
    URI _fileURI = this.getFileURI(model);
    String _string = _fileURI.toString();
    properties.put(EmfModel.PROPERTY_MODEL_URI, _string);
    String _plus = (Boolean.valueOf(readOnLoad) + "");
    properties.put(EmfModel.PROPERTY_READONLOAD, _plus);
    String _plus_1 = (Boolean.valueOf(storeOnDisposal) + "");
    properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, _plus_1);
    emfModel.load(properties, ((IRelativePathResolver) null));
    return emfModel;
  }
  
  private URI getFileURI(final String fileName) throws URISyntaxException {
    URL res = DNIRunner.class.getResource(fileName);
    URI binUri = res.toURI();
    URI uri = null;
    String _string = binUri.toString();
    int _indexOf = _string.indexOf("bin");
    boolean _greaterThan = (_indexOf > (-1));
    if (_greaterThan) {
      String _string_1 = binUri.toString();
      String _replaceAll = _string_1.replaceAll("bin", "src");
      URI _uRI = new URI(_replaceAll);
      uri = _uRI;
    } else {
      uri = binUri;
    }
    return uri;
  }
  
  public void saveOutput(final String txt, final String fname) {
    try {
      File file = new File(fname);
      file.createNewFile();
      FileWriter writer = new FileWriter(file);
      writer.write(txt);
      writer.flush();
      writer.close();
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
