package tools.descartes.dni.dnimm3.tools;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.junit.Assert;
import org.junit.Test;

import tools.descartes.dni.dnimm3ap.AdaptationPoints;
import tools.descartes.dni.dnimm3ap.impl.AdaptationPointsImpl;
import tools.descartes.dni.dnimm3ap.impl.DNIAPFactoryImpl;
import tools.descartes.dni.dnimm3ap.tools.DNIAPParser;

public class ParserTest {

	@Test
	public void shouldConvertFileToModel() {
		File file = new File("resources-test/scenario0B.dniap");
		AdaptationPointsImpl adaptationPoints = DNIAPParser.convertToModel(file);
		Assert.assertTrue(adaptationPoints.getRepositories().getNodes().size() > 0);
	}

	@Test
	public void shouldConvertFileToModelWithNetworkStructure() {
		File file = new File("resources-test/scenario0B.dniap");
		AdaptationPointsImpl adaptationPoints = DNIAPParser.convertToModel(file);
		Assert.assertEquals(6, adaptationPoints.getNetworkInfrastructure().getStructure().getNodes().size());
	}

	@Test
	public void shouldConvertModelToFile() throws IOException {
		DNIAPFactoryImpl factory = new DNIAPFactoryImpl();
		AdaptationPoints model = factory.createAdaptationPoints();
		File file = File.createTempFile("junit-dni-", ".dniap");
		file.deleteOnExit();
		DNIAPParser.writeToFile(model, file);
		Assert.assertTrue(file.length() > 100);
	}

	@Test
	public void shouldConvertModelToByteArray() throws IOException {
		DNIAPFactoryImpl factory = new DNIAPFactoryImpl();
		AdaptationPoints model = factory.createAdaptationPoints();
		byte[] bytes = DNIAPParser.convertToByteArray(model);
		Assert.assertTrue(bytes.length > 10);
	}

	@Test
	public void shouldConvertByteArrayToModel() throws IOException {
		Path path = Paths.get("resources-test/scenario0B.dniap");
		byte[] data = Files.readAllBytes(path);
		AdaptationPoints adaptationPoints = DNIAPParser.convertToModel(data);
		Assert.assertTrue(adaptationPoints.getRepositories().getNodes().size() > 0);
	}

}
