package tools.descartes.dni.dnimm3.tools;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.junit.Assert;
import org.junit.Test;

import tools.descartes.dni.dnimm3.NetworkInfrastructure;
import tools.descartes.dni.dnimm3.impl.DNIFactoryImpl;
import tools.descartes.dni.dnimm3.impl.NetworkInfrastructureImpl;

public class ParserTest {

	@Test
	public void shouldConvertFileToModel() {
		File file = new File("resources-test/scenario0A.dni");
		NetworkInfrastructureImpl networkInfrastructure = DNIParser.convertToModel(file);
		Assert.assertEquals(6, networkInfrastructure.getStructure().getNodes().size());
	}

	@Test
	public void shouldConvertFileWithAdaptationToModel() {
		File file = new File("resources-test/scenario0B.dni");
		NetworkInfrastructureImpl networkInfrastructure = DNIParser.convertToModel(file);
		Assert.assertEquals(6, networkInfrastructure.getStructure().getNodes().size());
	}

	@Test
	public void shouldConvertModelToFile() throws IOException {
		DNIFactoryImpl factory = new DNIFactoryImpl();
		NetworkInfrastructure model = factory.createNetworkInfrastructure();
		File file = File.createTempFile("junit-dni-", ".dni");
		file.deleteOnExit();
		DNIParser.writeToFile(model, file);
		Assert.assertTrue(file.length() > 100);
	}

	@Test
	public void shouldConvertModelToByteArray() throws IOException {
		DNIFactoryImpl factory = new DNIFactoryImpl();
		NetworkInfrastructure model = factory.createNetworkInfrastructure();
		byte[] bytes = DNIParser.convertToByteArray(model);
		Assert.assertTrue(bytes.length > 100);
	}

	@Test
	public void shouldConvertByteArrayToModel() throws IOException {
		Path path = Paths.get("resources-test/scenario0A.dni");
		byte[] data = Files.readAllBytes(path);
		NetworkInfrastructureImpl networkInfrastructure = DNIParser.convertToModel(data);
		Assert.assertEquals(6, networkInfrastructure.getStructure().getNodes().size());
	}

}
